@@ -833,7 +833,7 @@ char		buf[rtfBufSiz], *bp;
 		if (sp->rtfSNum < 0)
 		{
 			if (strncmp (buf, "Normal", 6) != 0
-				|| strncmp (buf, "Standard", 8) != 0)
+				&& strncmp (buf, "Standard", 8) != 0)
 				Error ("SSErr - missing style number");
 			sp->rtfSNum = 0;
 		}
