@@ -19,15 +19,24 @@
  *	1/ This copyright notice is preserved
  *	2/ You send me a copy of any changes for inclusion in a future release
  */
-#include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
+#ifdef __STDC__
+#   include <stdlib.h>
+#   include <unistd.h>
+#else /* !__STDC__ */
+#   include <malloc.h>
+#endif /* !__STDC__ */
+#ifndef SEEK_SET
+#   define SEEK_SET 0
+#endif
 #include "fonts.h"
 #include "r2L_version.h"
 #include "rtf2LaTeX.h"
 #include "rtf.h"
 
 
+char *progname;	/* Used in Error messages */
 RTFFuncPtr default_read_font = NULL;	/* default func to read style table */
 RTFFuncPtr default_read_style = NULL;	/* default func to read style table */
 /*****************************************************************************/
@@ -56,12 +65,22 @@ char c;
 char codefile[20],landfile[20];
 char *str = "12345.67";
 
+   progname = av[0];
    strcpy(codefile,"mac");
    strcpy(landfile,"english");
-   while((c = getopt(ac,av,"cC:dhHL:nprstT:uv:V")) != EOF)
+   while((c = getopt(ac,av,"cC:dhgGHL:npqrstT:uv:V")) != EOF)
       {
       switch (c)
        {
+       case 'g':
+	 german_squotes = 1;
+	 break;
+       case 'G':
+	 german_dquotes = 1;
+	 break;
+       case 'q':
+	 translate_quotes = 1;
+	 break;
        case 'C':
 	 strcpy(codefile,optarg);
 	 break;
@@ -99,7 +118,7 @@ char *str = "12345.67";
 	 smaller_cell_factor=atof(optarg);
 	 if (!(smaller_cell_factor > 0))
 		{
-		fprintf(stderr,"\nrtf2LaTeX: the argument of the -T flag must be greater than 0,\n but it is %g\n",smaller_cell_factor);
+		fprintf(stderr,"\n%s: Argument of the -T flag must be greater than 0,\n but it is %g\n", progname, smaller_cell_factor);
 		fprintf(stderr,"use -h for help\n");
 		exit(2);
 		}
@@ -111,17 +130,17 @@ char *str = "12345.67";
 	 verbose = atoi(optarg);
 	 if (verbose!=1 && verbose!=2)
 		{
-		fprintf(stderr,"\nrtf2LaTeX: The level of verbose can only be 1 or 2\n");
+		fprintf(stderr,"\n%s: The level of verbose can only be 1 or 2\n", progname);
 		fprintf(stderr,"use -h for help\n");
 		exit(2);
 		}
 	 break;
        case 'V':
-	 fprintf(stderr,"RTF2LaTeX: %s\n",version);
+	 fprintf(stderr,"%s: %s\n",progname,version);
 	 exit(0);
 	 break;
        case '?':
-	 fprintf(stderr,"rtf2LaTeX: ");
+	 fprintf(stderr,"%s: ", progname);
 	 usage();
 	 exit(2);
       }
@@ -132,14 +151,14 @@ char *str = "12345.67";
       	strcat(av[optind],".rtf");
       if((fil = fopen(av[optind],"r")) == NULL)
          {
-	 fprintf(stderr,"Can't open %s\n",av[optind]);
+	 fprintf(stderr,"%s: Can't open %s\n",progname,av[optind]);
 	 exit(1);
          }
       RTFSetStream(fil);
       }
   if (optind+1<ac)
 	{
-	fprintf(stderr,"rtf2LaTeX: after the filename (%s) there are no more arguments or flags allowed!\n",av[optind]);
+	fprintf(stderr,"%s: After the filename (%s) there are no more arguments or flags allowed!\n", progname, av[optind]);
 	fprintf(stderr,"use -h for help\n");
 	exit(2);
 	}
@@ -169,6 +188,8 @@ char *str = "12345.67";
 						e.g german.land    */
    print_text(header_erwinbet,stdout);
    output_str("\\documentstyle",'\0');
+   if(german_squotes || german_dquotes)
+       output_str("[german]",'\0');
    output_str("{article}\n",'\0');
    if(header_on)
 	{
@@ -188,12 +209,17 @@ char *str = "12345.67";
    RTFRead();
    
    if(rtf_group != 0) {
-      fprintf(stderr,"End of file is in an unclosed RTF group (level %d)\n",
-	      rtf_group);
+      fprintf(stderr,"%s: End of file is in an unclosed RTF group (level %d)\n",
+	      progname,rtf_group);
    }
    if(LaTeX_group != 0) {
-      fprintf(stderr,"End of file is in an unclosed LaTeX group (level %d)\n",
-	      LaTeX_group);
+      fprintf(stderr,"%s: End of file is in an unclosed LaTeX group (level %d)\n",
+	      progname,LaTeX_group);
+   }
+   if(quotecount % 2)
+   {
+      fprintf(stderr,"%s: Uneven number of quotes (%d) counted\n",
+	      progname, quotecount);
    }
    if(fil != NULL) fclose(fil);
    output_str("\\end{document}",'\0');
@@ -202,14 +228,14 @@ char *str = "12345.67";
 have a look in the man page section troubleshooting \n\n");
    return(0);
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Token class callbacks
  */
 static void
 UnknownClass()
 {
-   fprintf(stderr,"Unknown Token: %s\n",rtfTextBuf);
+   fprintf(stderr,"%s: Unknown Token: %s\n",progname,rtfTextBuf);
 }
 /*****************************************************************************/
 static void
@@ -227,7 +253,7 @@ GroupClass()
       break;
     case rtfEndGroup:
       if(--rtf_group == -1) {
-	 fprintf(stderr,"Unbalanced group\n");
+	 fprintf(stderr,"%s: Unbalanced group\n", progname);
       } else {
 	 while(pop_LaTeX_stack()) continue;
 	 pop_rtf_group();
@@ -259,8 +285,8 @@ TextClass()
 {
    output(rtfMajor,1);
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Process control symbol.
  */
 static void
@@ -280,10 +306,12 @@ ControlClass()
       Destination();
       break;
     case rtfFontFamily:			/* only occurs within font table */
-      fprintf(stderr,"You shouldn't see rtfFontType: minor %d\n",rtfMinor);
+      fprintf(stderr,"%s: You shouldn't see rtfFontType: minor %d\n",
+	progname,rtfMinor);
       break;
     case rtfColorName:			/* only occurs within color table */
-      fprintf(stderr,"You shouldn't see rtfColorName: minor %d\n",rtfMinor);
+      fprintf(stderr,"%s: You shouldn't see rtfColorName: minor %d\n",
+	progname,rtfMinor);
       break;
     case rtfStyleAttr:			/* only occurs within stylesheet */
       switch (rtfMinor) {
@@ -306,7 +334,8 @@ ControlClass()
 	 }
 	 break;
        default:
-	 fprintf(stderr,"Illegal minor number for StyleAttr: %d\n",rtfMinor);
+	 fprintf(stderr,"%s: Illegal minor number for StyleAttr: %d\n",
+	    progname,rtfMinor);
 	 break;
       }
       break;
@@ -330,7 +359,8 @@ ControlClass()
 	      CharAttr();
       break;
     case rtfPictAttr:
-      fprintf(stderr,"You shouldn't see rtfPictAttr: minor %d\n",rtfMinor);
+      fprintf(stderr,"%s: You shouldn't see rtfPictAttr: minor %d\n",
+	progname,rtfMinor);
       break;
     case rtfFieldAttr:
       FieldAttr();
@@ -343,8 +373,8 @@ ControlClass()
       break;
    }
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Control class major number handlers.  Each one switches on the
  * minor numbers that occur within the major number.  rtfStyleSheet,
  * rtfFontTbl, and rtfColorTbl are not in the switch because they're
@@ -496,9 +526,10 @@ Destination()
     case rtfIVersion:
       break;
     case rtfIVerscomm:
-/* \verscomm is a token, which make Macintosh's WORD.  *
- * I don't know the meaning,                           *
- * because it isn't specified in the RTF-specification.*/
+/* \verscomm is a token, made by Macintosh's WORD.
+ * I don't know the meaning,
+ * because it isn't specified in the RTF-specification.
+ */
        RTFSkipGroup();
        RTFUngetToken();
        break;
@@ -584,8 +615,10 @@ SpecialChar()
       if(RTFCheckCM(rtfGroup,rtfEndGroup) ||
 	 RTFCheckCMM(rtfControl,rtfParAttr,rtfParDef)) {
 	 end_of_par = 1;
-/*      } else if(in_table && RTFCheckCMM(rtfControl,rtfSpecialChar,rtfPar)) {
-	 end_table();			/* two \par's in a row */
+/* Don't know if this one is needed anymore
+ *      } else if(in_table && RTFCheckCMM(rtfControl,rtfSpecialChar,rtfPar)) {
+ *	 end_table();			* two \par's in a row *
+ */
       } else {
 	 end_para();
       }
@@ -680,6 +713,39 @@ SpecialChar()
       break;
     case rtfIIntID:
       break;
+    case rtfBullet:
+      output_str("\\bullet ",'\0');
+      break;
+    case rtfEmDash:
+      output_str("---",'\0');
+      break;
+    case rtfEnDash:
+      output_str("--",'\0');
+      break;
+    case rtfLQuote:
+      if(german_squotes)
+	 output_str("\\glq ",'\0');
+      else
+	 output_str("`",'\0');
+      break;
+    case rtfRQuote:
+      if(german_squotes)
+	 output_str("\\grq ",'\0');
+      else
+	 output_str("'",'\0');
+      break;
+    case rtfLDblQuote:
+      if(german_dquotes)
+	 output_str("\"`",'\0');
+      else
+	 output_str("``",'\0');
+      break;
+    case rtfRDblQuote:
+      if(german_dquotes)
+	 output_str("\"'",'\0');
+      else
+	 output_str("''",'\0');
+      break;
    }
 }
 static void
@@ -924,9 +990,10 @@ TblAttr()
       break;
       }
     case rtfCellShading:
-/* \clshdng is a token, which make Macintosh's WORD   *
- * I don't know the meaning,                          *
- * because it isn't specified in the RTF-specification*/
+/* \clshdng is a token, made by Macintosh's WORD
+ * I don't know the meaning,
+ * because it isn't specified in the RTF-specification
+ */
       break;
     case rtfMergeRngFirst:
       break;
@@ -1121,7 +1188,8 @@ ParAttr()
 	tab=1;
 	if(ntabs >= NTABS) {
 		if(ntabs == NTABS)
-			fprintf(stderr,"Attempt to set more than %d tabs\n",NTABS);
+			fprintf(stderr,"%s: Attempt to set more than %d tabs\n",
+			    progname,NTABS);
 	} else {
 		tabstops[ntabs++].pos = rtfParam;
 		tabstops[ntabs].type = TabLeft;
@@ -1272,8 +1340,8 @@ PosAttr()
       break;
    }
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Deal with Pict destinations
  */
 static void
@@ -1281,12 +1349,13 @@ read_pict()
 {
 static int count=0;
 if (verbose && count++==0)
-      fprintf(stderr,"rtf2LaTeX doesn't support any picture-things yet; be patient\n");
+      fprintf(stderr,"%s: Don't support any picture-things yet; be patient\n",
+	  progname);
 if (verbose>1)
 	msg_not_yet(rtfTextBuf+1);
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Deal with things like footnote numbering and page numbers; things
  * that involve the headline or footline
  */
@@ -1295,6 +1364,7 @@ set_headfoot_lines()
 {
    int pageno_top = (pageno_y < paper_height/2 ? 1 : 0);
    if(!change_headfoot) return;
+# ifdef notdef /* I think this is left over from rtf2tex */
 /*
  * The headline first
  *
@@ -1319,9 +1389,10 @@ set_headfoot_lines()
    output_str("}",'\n');
    change_headfoot = 0;			/* we've done it *
 */
+# endif /* notdef */
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Convert the pageno to the desired form
  */
 static char *
@@ -1338,8 +1409,8 @@ page_num()
       return("");
    }
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * This is called when we see the first text token after each \par
  */
 #define CURRENT(WHAT)			/* is WHAT up-to-date? */ \
@@ -1380,8 +1451,8 @@ end_para()
    end_of_par = 0;
    (void)RTFSetClassCallback(rtfText, start_para);
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Force an update of the current state, only emitting commands
  * that actually change anything.
  */
@@ -1478,8 +1549,8 @@ if (Cformatting_para)
    rtf_current.LaTeX_stack = NULL;
    rtf_current.prev = NULL;
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * ALLDONE
  */
 
@@ -1618,10 +1689,10 @@ CharAttr()
 }
 
 /*****************************************************************************
-/*
+ *
  * Various ways of refusing to deal with a keyword
  *
-/*
+ *
  * Treat \from as \to (e.g. treat \ul as \i)
  */
 static void
@@ -1654,10 +1725,13 @@ msg_not_supported(name)
 char *name;
 {
    static int pointer = 0;
-   for (pointer=0;(name[pointer]!='\0' && isalpha(name[pointer]));pointer++) continue;
-   name[pointer]='\0'; 		/*No argument will be printed*/
+   for (pointer=0;(name[pointer]!='\0' && isalpha(name[pointer]));pointer++)
+      continue;
+   /* Should fix problem writing to string constants with gcc */
+   if(name[pointer] != '\0')
+      name[pointer]='\0'; 		/*No argument will be printed*/
    if(verbose && !writing_defs) {
-      fprintf(stderr,"rtf2LaTeX doesn't support \\%s; sorry\n",name);
+      fprintf(stderr,"%s: Don't support \\%s; sorry\n",progname,name);
    }
 }
 /*
@@ -1668,15 +1742,15 @@ msg_not_yet(name)
 char *name;
 {
    if(verbose && !writing_defs) {
-      fprintf(stderr,"rtf2LaTeX doesn't support \\%s yet; be patient\n",name);
+      fprintf(stderr,"%s: Don't support \\%s yet; be patient\n",progname,name);
    }
 }
 
 
 
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Convert a string to a form that LaTeX can handle
  *
  * Remove spaces and capitalise the following letter, 
@@ -1713,7 +1787,6 @@ static void
 usage()
 {
    static char *msg[] = {
-      "Usage: rtf2LaTeX [options] [RTF-file]",
       "Your options are:",
       "       -c      No character formatting stuff",
       "       -C file Use another translation-file for characters above 128",
@@ -1738,11 +1811,12 @@ usage()
       NULL,
    };
 
+   fprintf(stderr, "Usage: %s [options] [RTF-file]\n", progname);
    print_text(msg,stderr);
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * print some text MSG to a stream FIL
  */
 static void
@@ -1757,8 +1831,8 @@ FILE *fil;
    }
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * set a Font
  */
 static void
@@ -1776,8 +1850,8 @@ char *start;				/* end the group that sets the font */
    }
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * change fontsize
  */
 set_fontsize(fontsize)
@@ -1858,8 +1932,8 @@ output_str("\\Huge ",'\0');
 }
 
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * set higher or lower text
  */
 
@@ -1875,7 +1949,8 @@ int param;
 	       {
 	       if(verbose)
 		  {
-		  fprintf(stderr,"Failed to find end of sub/superscript\n");
+		  fprintf(stderr,"%s: Failed to find end of sub/superscript\n",
+		    progname);
 		  }
 	       return;
 	       }
@@ -1905,8 +1980,8 @@ int param;
 	      push_LaTeX_stack(" \\egroup",Sub_Super,rtfParam);
 	      }
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * set smallcaps
  */
 
@@ -1923,8 +1998,8 @@ if (initialised)
 output_str("\\sc ",'\0');
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Print a string, ensuring that we are in math mode at the time
  */
 static void
@@ -1936,8 +2011,8 @@ char *str;
       output('$',0);
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Write a character, filling the the output text as we go. Characters
  * special to LaTeX are treated appropriately.
  */
@@ -1980,9 +2055,10 @@ int quote;				/* quote LaTeX's special chars? */
 	    
 	    str = (c < 128) ? symbol[c] : symbol8[c & '\177'];
 	    if(*str == '\0') {
-	       if(verbose) {
-		  fprintf(stderr,"Unknown Tech character: 0x%x\n",c);
-	       }
+	       /* if(verbose) { */
+		  fprintf(stderr,"%s: Unknown Tech character: 0x%x\n",
+		    progname,c);
+	       /* } */
 	    }
 	    if(verbose) {
 		  fprintf(stderr,"A Tech character: 0x%x\n",c);
@@ -2003,8 +2079,11 @@ int quote;				/* quote LaTeX's special chars? */
 	    output_str("$^{\\wedge}$",'\0');
 	    return;
 	  case '"':
-	    fputc('`',stdout);
-	    c = '`';
+	    if(translate_quotes)
+		c = (quotecount++ % 2) ? '\'' : '`';
+	    else
+		c = '`';
+	    fputc(german_dquotes ? '"' : c, stdout);
 	    column++;
 	    break;
 	  case '<': case '>':
@@ -2032,7 +2111,8 @@ int quote;				/* quote LaTeX's special chars? */
 	    if(!isalnum(c) && !ispunct(c))
 	       {
 	       if (verbose)
-		 fprintf(stderr,"rtf2LaTeX: Unknown 7-bit character: 0x%x\n",c);
+		  fprintf(stderr,"%s: Unknown 7-bit character: 0x%x\n",
+		     progname,c);
 	       return;
 	       }
 	    break;
@@ -2042,8 +2122,8 @@ int quote;				/* quote LaTeX's special chars? */
 fputc(c,stdout);
 column++;
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Output an entire word, without trying to quote any special characters.
  * The character C is then output using output(), which allows proper page
  * breaks.
@@ -2072,8 +2152,8 @@ if (str[len-1] == '\n')
 	column=0;
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Deal with special characters above \177.
  */
 
@@ -2088,8 +2168,8 @@ struct Char8bit *next;
  char8 *help;
 
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  *   write a 8bit character
  */
 
@@ -2112,16 +2192,17 @@ else
       if(verbose)
 	 {
 	 static count=0;
-	 fprintf(stderr,"Unknown 8-bit character: 0x%x\n",c);
+	 fprintf(stderr,"%s: Unknown 8-bit character: 0x%x\n",progname,c);
 	 if (count++<1||verbose>1)
 		fprintf(stderr,"   Have a look in the file .cod\n");
 	 }
       }
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  *   read the file mac.code (or an other file e.g. ansi.code: use flag -C) 
- *   and store the commands for characters with the 8bit set in the list "list_8bit"
+ *   and store the commands for characters with the 8bit set in the list
+ *   "list_8bit"
  */
 
 
@@ -2136,26 +2217,26 @@ int pos;
 open_code_file(codefile);
 if (fgets(line,line_length,fpcode)==NULL)
 	{
-	fprintf(stderr,"\nincorrect file a8bit    \
+	fprintf(stderr,"\n%s: Incorrect file a8bit    \
 			\nat first there must be a number of the ascii character with msb set  \
-			\n(between 128 and 255)");
+			\n(between 128 and 255)\n",progname);
 	exit(1);
 	}
 	for (pos=23;line[pos]==' ';pos--);
-	line[pos+1]='\0';			/* cut the commentar in the file*/
+	line[pos+1]='\0';			/* cut the comment in the file*/
 	list_8bit=(struct Char8bit *)malloc((int)sizeof(struct Char8bit));
 	if (list_8bit==(struct Char8bit *)NULL)
 		{
-		fprintf(stderr,"TCErr - cannot allocate a 8bit-character name\n");
+		fprintf(stderr,"%s: TCErr - cannot allocate a 8bit-character name\n", progname);
 		exit(1);
                 }
 	sscanf(line,"%d",&(list_8bit->nr));
 	list_8bit->str=StrSave(line+4);
 	if (list_8bit->nr<128 || list_8bit->nr>255)
 		{
-		fprintf(stderr,"\nincorrect file a8bit  \
+		fprintf(stderr,"\n%s: Incorrect file a8bit  \
 				\nat first there must be a number of the ascii character with msb set  \
-				\n(between 128 and 255)");
+				\n(between 128 and 255)\n", progname);
 		exit(1);
 		}
 	list_8bit->next=NULL;
@@ -2166,7 +2247,7 @@ while (fgets(line,line_length,fpcode)!=NULL)
 	help=(struct Char8bit *)malloc((int)sizeof(struct Char8bit));
 	if (help==(struct Char8bit *)NULL)
 		{
-		fprintf(stderr,"SSErr - cannot allocate a 8bit-character name\n");
+		fprintf(stderr,"%s: SSErr - cannot allocate a 8bit-character name\n", progname);
 		exit(1);
                 }
 	help->next=list_8bit;
@@ -2175,9 +2256,9 @@ while (fgets(line,line_length,fpcode)!=NULL)
 	list_8bit->str=StrSave(line+4);
 	if (list_8bit->nr<128 || list_8bit->nr>255)
 		{
-		fprintf(stderr,"\nincorrect file a8bit  \
+		fprintf(stderr,"\n%s: Incorrect file a8bit  \
 				\nat first there must be a number of the ascii character with msb set  \
-				\n(between 128 and 255)");
+				\n(between 128 and 255)\n", progname);
 		exit(1);
                 }
 	}
@@ -2185,17 +2266,16 @@ if (fpcode!=NULL) fclose(fpcode);
 }
 
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * write an errormassage to stderr
  *
  */
 static void
 p6(proc_name)
-char *proc_name;	/*name of ther procedure where the mistake must be
-                 */
+char *proc_name;	/*name of ther procedure where the mistake must be */
 {
-     fprintf(stderr," rtf2LaTeX: mistake in procedure %s\n",proc_name);
+     fprintf(stderr," %s: Problem in procedure %s\n",progname,proc_name);
      fprintf(stderr,"           please call Erwin Wechtl\n");
      fprintf(stderr,"                       W""ortgasse 2/18\n");
      fprintf(stderr,"                       A-2500 Baden\n");
@@ -2203,8 +2283,8 @@ char *proc_name;	/*name of ther procedure where the mistake must be
      fprintf(stderr,"                       Tel.: 43/2252/44686\n");
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * set end af table
  */
 
@@ -2223,8 +2303,8 @@ end_table()
 }
 
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  *  translate each RTF-style into a new LaTeX environment
  *  and look for special WORD styles like heading (tranlated into section)
  *  or footer (ignored)
@@ -2244,7 +2324,7 @@ for(rtfstyle=RTFGetStyle(-1);rtfstyle!=(RTFStyle *)NULL;rtfstyle=rtfstyle->rtfNe
    LaTeXstyle=(LATEXStyle *)malloc(sizeof(struct LATEXStyle));
    if (LaTeXstyle==NULL)
 	{
-	fprintf(stderr,"\nrtf2LaTeX: can't alloc LaTeXstyle");
+	fprintf(stderr,"\n%s: Can't alloc LaTeXstyle\n",progname);
 	exit(1);
 	}
    LaTeXstyle->stacktype=Undefined;
@@ -2296,8 +2376,8 @@ for(rtfstyle=RTFGetStyle(-1);rtfstyle!=(RTFStyle *)NULL;rtfstyle=rtfstyle->rtfNe
 if (fpland!=NULL) fclose(fpland);
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  *   begin a style environment
  */
 
@@ -2320,14 +2400,17 @@ char buf[80];
 		rtfstyle=RTFGetStyle(n);
 		if (verbose>1 || (verbose && count++==0))
 			{
-			fprintf(stderr,"There are some troubles with the style %s with the number %d, \n  so this style is ignored\n",rtfstyle->rtfSName,n);
+			fprintf(stderr,"%s: There are some troubles with the style %s with the number %d, \n  so this style is ignored\n",
+			    progname,rtfstyle->rtfSName,n);
 			}
+# ifdef notdef /* Don't know if this is needed anymore */
 /*		sprintf(buf,"\n%% The Style %s with the number %d may begin here\n",rtfstyle->rtfSName,n);
 		output_str(buf,'\0');
 		output_str("%%But rtf2LaTeX has had some troubles with it",'\0');
 		sprintf(buf,"\n%  The Style %s with the number %d may end here\n",rtfstyle->rtfSName,n);
 		push_LaTeX_stack(buf,NormalWORDstyle,0);
 */
+# endif /* notdef */
 		return;
 		}
 	if (LaTeXstyle->special_WinwordStyle)	/* if heading,footer, ... */
@@ -2350,9 +2433,10 @@ char buf[80];
 	push_LaTeX_stack(LaTeXstyle->EndCommand,LaTeXstyle->stacktype,0);
 }
 
-/*****************************************************************************/
-/*
- *  look in the file english.land (or an other file e.g. german.land: use flag -L) 
+/*****************************************************************************
+ *
+ *  look in the file english.land (or an other file e.g. german.land:
+ *  use flag -L) 
  *  if this is a Spezial WORD style like heading, footer, ...
  */
 
@@ -2385,8 +2469,8 @@ while (fgets(line,line_length,fpland)!=NULL)
 return(0);
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  *   malloc for a string
  */
 
@@ -2397,13 +2481,13 @@ char	*s;
 char	*p;
 
 	if ((p = (char *)malloc (strlen (s) + 1)) == (char *) NULL)
-		fprintf(stderr,"malloc\n");
+		fprintf(stderr,"%s: Error in malloc\n",progname);
 	return (strcpy (p, s));
 }
 
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  *   set the position of the Tabstops and output the begin environment
  */
 
@@ -2441,8 +2525,8 @@ start_tabstops()
    tab=2;
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  *   initialise the array of the Tabstops-positions
  */
 
@@ -2459,8 +2543,8 @@ for (i=0;i<NTABS;i++)
 	ntabs=0;
 }
 
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Stack stuff -- RTF and LaTeX grouping go on separate stacks
  *
  * pop the status stack, putting the free'd element on the free list
@@ -2472,7 +2556,7 @@ pop_rtf_group()
    RTF_STACK *temp;
    
    if(rtf_ptr->prev == NULL) {
-      fprintf(stderr,"Attempt to pop an empty stack\n");
+      fprintf(stderr,"%s: Attempt to pop an empty stack\n", progname);
       abort();
    }
    temp = rtf_ptr->prev;
@@ -2493,7 +2577,7 @@ push_rtf_group()
       rtf_free_list = rtf_free_list->prev;
    } else {
       if((temp = (RTF_STACK *)malloc(sizeof(RTF_STACK))) == NULL) {
-	 fprintf(stderr,"Can't allocate storage for stack\n");
+	 fprintf(stderr,"%s: Can't allocate storage for stack\n",progname);
 	 exit(1);
       }
    }
@@ -2502,8 +2586,8 @@ push_rtf_group()
    rtf_ptr = temp;
    rtf_ptr->LaTeX_stack = NULL;
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Now the LaTeX stacks. Use the stack in the current rtf_ptr frame
  */
 static LATEX_STACK *myTeX_free_list = NULL;
@@ -2546,7 +2630,7 @@ long flags;				/* and corresponding flags */
       myTeX_free_list = myTeX_free_list->prev;
    } else {
       if((temp = (LATEX_STACK *)malloc(sizeof(LATEX_STACK))) == NULL) {
-	 fprintf(stderr,"Can't allocate storage for stack\n");
+	 fprintf(stderr,"%s: Can't allocate storage for stack\n",progname);
 	 exit(1);
       }
    }
@@ -2558,8 +2642,8 @@ long flags;				/* and corresponding flags */
    rtf_ptr->LaTeX_stack->flags = flags;
    return;
 }
-/*****************************************************************************/
-/*
+/*****************************************************************************
+ *
  * Return the flags of the proper type for the top element of the LaTeX stack
  */
 int
@@ -2586,13 +2670,14 @@ char c;
       	strcat(filename,".code");
       if((fpcode = fopen(filename,"r")) == NULL)
 	 {
-	 fprintf(stderr,"Can't open %s\n",filename);
+	 fprintf(stderr,"%s: Can't open %s\n",progname,filename);
 	 exit(1);
 	 }
 if (fgets(line,line_length,fpcode)==NULL)
 	{
-	fprintf(stderr,"\nincorrect file a8bit   \
-			\nthere must be a commentar in the first line\n");
+	fprintf(stderr,"\n%s: Incorrect file a8bit   \
+			\nthere must be a comment in the first line\n",
+			progname);
 	exit(1);
 	}
 }
@@ -2606,7 +2691,7 @@ char landfile[20];
       if ((strchr(landfile,'.')) == NULL)
       	strcat(filename,".land");
       if((fpland = fopen(filename,"r")) == NULL) {
-	 fprintf(stderr,"Can't open %s\n",filename);
+	 fprintf(stderr,"%s: Can't open %s\n",progname,filename);
 	 exit(1);
       }
 }
